unit ChkStrEd;

interface

uses
  {$IFDEF WIN32}
  Windows,
  {$ELSE}
  WinTypes, WinProcs,
  {$ENDIF}
  SysUtils, Messages, Classes, Graphics, Controls,
  Forms, StdCtrls, Buttons, IniFiles, DsgnIntf, Menus, Grids, Dialogs,
  ExtCtrls;

type
  TCheckStringListProperty = class( TPropertyEditor )
    function GetAttributes : TPropertyAttributes; override;
    function GetValue: string; override;
    procedure Edit; override;
  end;

  TChkStrLstDlg = class( TForm )
    pnlToolbar: TPanel;
    btnCut: TSpeedButton;
    btnCopy: TSpeedButton;
    btnPaste: TSpeedButton;
    btnUndo: TSpeedButton;
    btnFont: TSpeedButton;
    pnlStringList: TPanel;
    lblCount: TLabel;
    lblLineNr: TLabel;
    lblLine: TLabel;
    mnuEdit: TPopupMenu;
    mnuUndo: TMenuItem;
    mnuCut: TMenuItem;
    mnuCopy: TMenuItem;
    mnuPaste: TMenuItem;
    dlgFont: TFontDialog;
    btnOk: TButton;
    btnCancel: TButton;
    chkDefault: TCheckBox;
    grdEdit: TStringGrid;
    btnDelete: TSpeedButton;
    cmdDown: TSpeedButton;
    cmdUp: TSpeedButton;
    edtStrings: TMemo;
    procedure FormCreate( Sender : TObject );
    procedure FormDestroy( Sender : TObject );
    procedure btnOkClick( Sender : TObject );
    procedure btnFontClick( Sender : TObject );
    procedure btnUndoClick( Sender : TObject );
    procedure btnCutClick( Sender : TObject );
    procedure btnCopyClick( Sender : TObject );
    procedure btnPasteClick( Sender : TObject );
    procedure grdEditClick(Sender: TObject);
    procedure grdEditDrawCell(Sender: TObject; Col, Row: Longint;
      Rect: TRect; State: TGridDrawState);
    procedure grdEditDblClick(Sender: TObject);
    procedure grdEditKeyPress(Sender: TObject; var Key: Char);
    procedure grdEditMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure grdEditMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure btnDeleteClick(Sender: TObject);
    procedure cmdDownClick(Sender: TObject);
    procedure cmdUpClick(Sender: TObject);
    procedure grdEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    DelphiIni : TIniFile;
    FPropName : string;
    procedure UpdateLineColStatus;
    procedure UpdateClipboardStatus;
    procedure EnableButtons( Enable : Boolean );
    procedure ClearRow(Row: Integer);
    procedure ResizeGrid;
  public
    procedure SetTextToGrid;
    procedure SetGridToText;
  end;


implementation

{$R *.DFM}

uses
  ClipBrd, Printers, ChkList;

const
  Section = 'SheAr.ChkStrListEditor';

  fsBoldMask      = 8;
  fsItalicMask    = 4;
  fsUnderlineMask = 2;
  fsStrikeOutMask = 1;
  fsNormal        = 0;


{ TCheckStringListProperty Methods }

function TCheckStringListProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [ paReadOnly, paDialog ];                  { Edit method will display a dialog }
end;

function TCheckStringListProperty.GetValue : string;
begin
  { The GetPropType method is used to retrieve information pertaining to the   }
  { property type being edited.  In this case, the Name of the property class  }
  { is displayed in the value column of the Object Inspector.                  }

  Result := Format( '(%s)', [ GetPropType^.Name ] );
end;

procedure TCheckStringListProperty.Edit;
var
  Dialog : TChkStrLstDlg;
  Cls    : TCheckListStrings;
  i      : Integer;

begin
  Dialog := TChkStrLstDlg.Create( Application );
  try
    if PropCount = 1 then
    begin
      Dialog.FPropName := GetComponent(0).Owner.Name + '.' +
                          GetComponent(0).Name + '.' + GetName;
      Dialog.Caption :=  Dialog.FPropName + ' - ' + Dialog.Caption;
    end;

    { Copy string list of property into the memo of the dialog }
    Cls := TCheckListStrings(GetOrdValue);
    for i:=0 to Cls.Count-1 do
     case Cls.State[i] of
       csUnchecked: Dialog.edtStrings.Lines.Add('0|'+Cls.Strings[i]+'|'+
                    IntToStr(Integer(Cls.Enabled[i]))+'|'+IntToStr(Cls.Tag[i]));
       csChecked  : Dialog.edtStrings.Lines.Add('1|'+Cls.Strings[i]+'|'+
                    IntToStr(Integer(Cls.Enabled[i]))+'|'+IntToStr(Cls.Tag[i]));
       csGrayed   : Dialog.edtStrings.Lines.Add('2|'+Cls.Strings[i]+'|'+
                    IntToStr(Integer(Cls.Enabled[i]))+'|'+IntToStr(Cls.Tag[i]));
     end;

    Dialog.SetTextToGrid;
    Dialog.UpdateLineColStatus;
    if Dialog.ShowModal = mrOK then
     SetOrdValue( Longint( Dialog.edtStrings.Lines ) );
  finally
    Dialog.Free;
  end;
end;

type
    TNewStrGrid = class(TStringGrid); { Needed to access InPlaceEditor }

{ TChkStrLstDlg Methods }
function GetItem(Item: String; Index: Integer; Sep: Char): String;
var
  iPos, iCount: Integer;
begin
  Result := '';
  iCount := 1;
  iPos := Pos(Sep, Item);
  while (iPos > 0) and (iCount < Index) do
   begin
     inc(iCount);
     System.Delete(Item, 1, iPos);
     iPos := Pos(Sep, Item);
   end;
  if iCount = Index then
   begin
     if iPos = 0 then
      Result := Item
     else
      Result := Copy(Item, 1, iPos-1);
   end;
end;

procedure TChkStrLstDlg.ClearRow(Row : Integer);
begin
  grdEdit.Cells[0, Row] := '0';
  grdEdit.Cells[1, Row] := '';
  grdEdit.Cells[2, Row] := '1';
  grdEdit.Cells[3, Row] := '0';
end;

procedure TChkStrLstDlg.SetTextToGrid;
var
   sText : string;
   i, j  : integer;
begin
  grdEdit.RowCount := edtStrings.Lines.Count+2;
  for i:=0 to edtStrings.Lines.Count-1 do
   begin
     sText := edtStrings.Lines[i];
     for j:=0 to 3 do
      grdEdit.Cells[j, i+1] := GetItem(sText, j+1, '|');
   end;
  ClearRow(grdEdit.RowCount-1);
  ResizeGrid;
end;

procedure TChkStrLstDlg.SetGridToText;
var
   i : Integer;
begin
  edtStrings.Clear;
  for i:=1 to grdEdit.RowCount-2 do
   edtStrings.Lines.Add(grdEdit.Cells[0, i]+'|'+grdEdit.Cells[1, i]+'|'+grdEdit.Cells[2,i]+'|'+grdEdit.Cells[3,i]);
end;

procedure TChkStrLstDlg.ResizeGrid;
begin
  with grdEdit do
   if VisibleRowCount<RowCount-1 then
     ColWidths[1] := Width - ColWidths[0] - ColWidths[2] - ColWidths[3] - 8 - GetSystemMetrics(SM_CYHSCROLL)
   else
     ColWidths[1] := Width - ColWidths[0] - ColWidths[2] - ColWidths[3] - 8;
end;

procedure TChkStrLstDlg.FormCreate(Sender: TObject);
var
  StyleBits  : Byte;
begin
  { Load settings from DELPHI.INI File }
  DelphiIni := TIniFile.Create( 'DELPHI.INI' );
  with grdEdit.Font do
  begin
    Name := DelphiIni.ReadString( Section, 'FontName', 'MS Sans Serif' );
    Size := DelphiIni.ReadInteger( Section, 'FontSize', 8 );
    Color := DelphiIni.ReadInteger( Section, 'FontColor', clBlack );
    StyleBits := DelphiIni.ReadInteger( Section, 'FontStyle', fsNormal );
    Style := [];
    if StyleBits and fsBoldMask = fsBoldMask then
      Style := Style + [ fsBold ];
    if StyleBits and fsItalicMask = fsItalicMask then
      Style := Style + [ fsItalic ];
    if StyleBits and fsUnderlineMask = fsUnderlineMask then
      Style := Style + [ fsUnderline ];
    if StyleBits and fsStrikeOutMask = fsStrikeOutMask then
      Style := Style + [ fsStrikeOut ];
  end;
  grdEdit.Cells[0,0] := 'State';
  grdEdit.Cells[1,0] := 'Text';
  grdEdit.Cells[2,0] := 'Enabled';
  grdEdit.Cells[3,0] := 'Tag';
end;


procedure TChkStrLstDlg.FormDestroy(Sender: TObject);
begin
  DelphiIni.Free;
end;


procedure TChkStrLstDlg.btnOkClick(Sender: TObject);
var
  StyleBits : Byte;
begin
  if ChkDefault.Checked then
  begin                                            { Save New Default Settings }
    with grdEdit.Font do
    begin
      DelphiIni.WriteString( Section, 'FontName', Name );
      DelphiIni.WriteInteger( Section, 'FontSize', Size );
      DelphiIni.WriteInteger( Section, 'FontColor', Color );

      StyleBits := 0;
      if fsBold in Style then
        StyleBits := fsBoldMask;
      if fsItalic in Style then
        StyleBits := StyleBits + fsItalicMask;
      if fsUnderline in Style then
        StyleBits := StyleBits + fsUnderlineMask;
      if fsStrikeOut in Style then
        StyleBits := StyleBits + fsStrikeOutMask;
      DelphiIni.WriteInteger( Section, 'FontStyle', StyleBits );
    end;
  end;
  SetGridToText;
end;

procedure TChkStrLstDlg.btnFontClick(Sender: TObject);
begin
  dlgFont.Font := grdEdit.Font;
  if dlgFont.Execute then
  begin
    grdEdit.Font := dlgFont.Font;
  end;
end;

procedure TChkStrLstDlg.btnUndoClick(Sender: TObject);
begin
  TNewStrGrid(grdEdit).InPlaceEditor.Perform( wm_Undo, 0, 0 );
end;

procedure TChkStrLstDlg.btnCutClick(Sender: TObject);
begin
  TNewStrGrid(grdEdit).InplaceEditor.CutToClipboard;
  UpdateClipboardStatus;
end;

procedure TChkStrLstDlg.btnCopyClick(Sender: TObject);
begin
  TNewStrGrid(grdEdit).InplaceEditor.CopyToClipboard;
  UpdateClipboardStatus;
end;

procedure TChkStrLstDlg.btnPasteClick(Sender: TObject);
begin
  TNewStrGrid(grdEdit).InplaceEditor.PasteFromClipboard;
end;

procedure TChkStrLstDlg.UpdateLineColStatus;
begin
  lblLine.Caption := IntToStr( grdEdit.Row );
  lblCount.Caption := IntToStr( grdEdit.RowCount-1) + ' Line(s)';
  UpdateClipboardStatus;
end;

procedure TChkStrLstDlg.UpdateClipboardStatus;
var
  HasText      : Boolean;
  HasSelection : Boolean;
begin
  if TNewStrGrid(grdEdit).InplaceEditor<>nil then
   HasSelection := TNewStrGrid(grdEdit).InPlaceEditor.SelLength <> 0
  else
   HasSelection := False;
  BtnCut.Enabled := HasSelection;           { Cut and Copy are only enabled if }
  MnuCut.Enabled := HasSelection;            { the user has selected some text }
  BtnCopy.Enabled := HasSelection;
  MnuCopy.Enabled := HasSelection;
  HasText := Clipboard.HasFormat(CF_TEXT);
  BtnPaste.Enabled := HasText;                  { Paste is only enabled if the }
  MnuPaste.Enabled := HasText;                  { Clipboard contains Text      }
end;


procedure TChkStrLstDlg.EnableButtons( Enable : Boolean );
var
  SysMenu : HMenu;
begin
  btnUndo.Enabled := Enable;
  btnFont.Enabled := Enable;
  btnOK.Enabled := Enable;
  btnCancel.Enabled := Enable;
  chkDefault.Enabled := Enable;

  edtStrings.Enabled := Enable;

  btnCut.Enabled := Enable;
  btnCopy.Enabled := Enable;
  btnPaste.Enabled := Enable;
  if Enable then
    UpdateClipboardStatus;

  { Disable the Close menu item, so dialog cannot be closed }
  SysMenu := GetSystemMenu(Handle, False);
  if Enable then
    EnableMenuItem(SysMenu, SC_CLOSE, MF_BYCOMMAND or MF_ENABLED)
  else
    EnableMenuItem(SysMenu, SC_CLOSE, MF_BYCOMMAND or MF_DISABLED or MF_GRAYED);
end;

procedure TChkStrLstDlg.grdEditClick(Sender: TObject);
begin
  with grdEdit do
   begin
     if (Col=1) or (Col=3) then
      begin
        Options := Options + [goEditing];
        EditorMode := True;
      end
     else
      Options := Options - [goEditing];
   end;
  UpdateLineColStatus;
end;

procedure TChkStrLstDlg.grdEditDrawCell(Sender: TObject; Col, Row: Longint;
  Rect: TRect; State: TGridDrawState);
var
   FMinWidth : Integer;
   RectCheck : TRect;
   Halfy, i  : Integer;
begin
  if (Col=0) and (Row>0) then
   begin
     FMinWidth := 13;
     CopyRect(RectCheck, Rect);
     RectCheck.left := ((Rect.right-Rect.left) - FMinWidth) div 2;
     RectCheck.top := Rect.top + ((Rect.bottom-Rect.top) - FMinWidth) div 2;
     RectCheck.bottom := RectCheck.top + FMinWidth;
     RectCheck.right := RectCheck.left + FMinWidth;
     with grdEdit.Canvas, RectCheck do
      begin
        FillRect(Rect);
        Pen.Color := clBtnShadow;
        if grdEdit.Cells[Col, Row]='2' then
         Brush.Color := clBtnFace
        else
          Brush.Color := clWindow;
        Rectangle(left+1, top+1, right-1, bottom-1);
        if (grdEdit.Cells[Col, Row]='1') or (grdEdit.Cells[Col, Row]='2') then
         begin
           InflateRect(RectCheck, -3, -3);
           Pen.Color := clBlack;
           Pen.Width := 1;
           halfy := top+(bottom-top) div 2 + 1;
           for i:=0 to 2 do
            begin
              PolyLine([Point(left,halfy-i), Point(left+2, halfy+2-i)]);
              PolyLine([Point(left+2, halfy+2-i), Point(left+7, halfy-3-i)]);
            end;
         end
      end
   end
  else if (Col=2) and (Row>0) then
   begin
     if (grdEdit.Cells[Col, Row]='0') then
      grdEdit.Canvas.TextOut(Rect.left, Rect.top, 'False')
     else
      grdEdit.Canvas.TextOut(Rect.left, Rect.top, 'True');
   end;
end;

procedure TChkStrLstDlg.grdEditDblClick(Sender: TObject);
begin
  if grdEdit.Col=0 then
   case grdEdit.Cells[0, grdEdit.Row][1] of
    '0' : grdEdit.Cells[0, grdEdit.Row] := '1';
    '1' : grdEdit.Cells[0, grdEdit.Row] := '2';
    '2' : grdEdit.Cells[0, grdEdit.Row] := '0';
   end
  else if grdEdit.Col=2 then
   begin
     if grdEdit.Cells[2, grdEdit.Row]='0' then
      grdEdit.Cells[2, grdEdit.Row] := '1'
     else
      grdEdit.Cells[2, grdEdit.Row] := '0';
   end;
end;

procedure TChkStrLstDlg.grdEditKeyPress(Sender: TObject; var Key: Char);
begin
  if (grdEdit.Col=0) or (grdEdit.Col=2) then
   begin
     Key := #0;
     Exit;
   end;
  if grdEdit.Row=grdEdit.RowCount-1 then
   begin
     grdEdit.RowCount := grdEdit.RowCount+1;
     ClearRow(grdEdit.RowCount-1);
     UpdateLineColStatus;
     ResizeGrid;
   end;
end;

procedure TChkStrLstDlg.grdEditMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  UpdateClipboardStatus;
end;

procedure TChkStrLstDlg.grdEditMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  UpdateClipboardStatus;
end;

procedure TChkStrLstDlg.btnDeleteClick(Sender: TObject);
var
   i, j : integer;
begin
  if grdEdit.RowCount>2 then
   begin
     for i:=grdEdit.Row to grdEdit.RowCount-1 do
      for j:=0 to 3 do
       grdEdit.Cells[j, i] := grdEdit.Cells[j, i+1];
     grdEdit.RowCount := grdEdit.RowCount-1;
   end;
  ClearRow(grdEdit.RowCount-1);
  UpdateLineColStatus;
end;

procedure TChkStrLstDlg.cmdDownClick(Sender: TObject);
var
   sTmp    : string;
   i, iRow : integer;
begin
  iRow := grdEdit.Row;
  if iRow<grdEdit.RowCount-1 then
   begin
     for i := 0 to 3 do
      begin
        sTmp := grdEdit.Cells[i, iRow];
        grdEdit.Cells[i, iRow] := grdEdit.Cells[i, iRow+1];
        grdEdit.Cells[i, iRow+1] := sTmp;
      end;
     grdEdit.Row := iRow+1;
     UpdateLineColStatus;
   end;
end;

procedure TChkStrLstDlg.cmdUpClick(Sender: TObject);
var
   sTmp    : string;
   i, iRow : integer;
begin
  iRow := grdEdit.Row;
  if iRow>1 then
   begin
     for i:=0 to 3 do
      begin
        sTmp := grdEdit.Cells[i, iRow];
        grdEdit.Cells[i, iRow] := grdEdit.Cells[i, iRow-1];
        grdEdit.Cells[i, iRow-1] := sTmp;
      end;
     grdEdit.Row := iRow-1;
     UpdateLineColStatus;
   end;
end;

procedure TChkStrLstDlg.grdEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (grdEdit.Col=0) or (grdEdit.Col=2) then Key:=0;
end;

end.

